@@ -205,8 +205,8 @@ void add_cvs_excludes(void)
     add_exclude(cvs_ignore_list[i]);
 
   if ((p=getenv("HOME")) && strlen(p) < (MAXPATHLEN-12)) {
-    sprintf(fname,"%s/.cvsignore",p);
-    add_exclude_file(fname,0);
+	  slprintf(fname,sizeof(fname)-1, "%s/.cvsignore",p);
+	  add_exclude_file(fname,0);
   }
 
   add_exclude_line(getenv("CVSIGNORE"));
